=== modified file 'README.rst'
--- README.rst	2019-02-11 20:37:27 +0000
+++ README.rst	2020-03-22 01:35:14 +0000
@@ -18,8 +18,8 @@
 Breezy is a friendly fork of the Bazaar (``bzr``) project, hosted on
 http://bazaar.canonical.com/. It is backwards compatibility with
 Bazaar's disk format and protocols. One of the key differences with
-Bazaar is that Breezy also runs on Python 3.3 and later, in addition to
-Python 2.7.
+Bazaar is that Breezy runs on Python 3.3 and later, rather than on
+Python 2.
 
 Breezy highlights
 =================

=== modified file 'breezy/git/config.py'
--- breezy/git/config.py	2020-02-18 01:57:45 +0000
+++ breezy/git/config.py	2020-03-22 01:35:14 +0000
@@ -44,19 +44,62 @@
         return self._get_best_value('_get_user_id')
 
 
+class GitConfigSectionDefault(config.Section):
+    """The "default" config section in git config file"""
+
+    def __init__(self, config):
+        self._config = config
+
+    def get(self, name, default=None, expand=True):
+        if name == 'email':
+            try:
+                email = self._config.get((b'user', ), b'email')
+            except KeyError:
+                return None
+            try:
+                name = self._config.get((b'user', ), b'name')
+            except KeyError:
+                return email.decode()
+            return '%s <%s>' % (name.decode(), email.decode())
+        if name == 'gpg_signing_key':
+            try:
+                key = self._config.get((b'user', ), b'signingkey')
+            except KeyError:
+                return None
+            return key.decode()
+        return None
+
+
+class GitConfigStore(config.Store):
+    """Store that uses gitconfig."""
+
+    def __init__(self, config):
+        self._config = config
+
+    def get_sections(self):
+        return [
+            (self, GitConfigSectionDefault(self._config)),
+            ]
+
+
 class GitBranchStack(config._CompatibleStack):
     """GitBranch stack."""
 
     def __init__(self, branch):
+        section_getters = [self._get_overrides]
         lstore = config.LocationStore()
         loc_matcher = config.LocationMatcher(lstore, branch.base)
+        section_getters.append(loc_matcher.get_sections)
         # FIXME: This should also be looking in .git/config for
         # local git branches.
+        git = getattr(branch.repository, '_git', None)
+        if git:
+            cstore = GitConfigStore(git.get_config())
+            section_getters.append(cstore.get_sections)
         gstore = config.GlobalStore()
+        section_getters.append(gstore.get_sections)
         super(GitBranchStack, self).__init__(
-            [self._get_overrides,
-             loc_matcher.get_sections,
-             gstore.get_sections],
+            section_getters,
             # All modifications go to the corresponding section in
             # locations.conf
             lstore, branch.base)

=== modified file 'breezy/git/tests/test_blackbox.py'
--- breezy/git/tests/test_blackbox.py	2020-02-18 01:57:45 +0000
+++ breezy/git/tests/test_blackbox.py	2020-03-22 01:35:14 +0000
@@ -424,6 +424,37 @@
         self.assertEqual(out, '')
         self.assertTrue(err.endswith, '3 objects\n')
 
+    def test_local_whoami(self):
+        r = GitRepo.init("gitr", mkdir=True)
+        self.build_tree_contents([('gitr/.git/config', """\
+[user]
+  email = some@example.com
+  name = Test User
+""")])
+        out, err = self.run_bzr(["whoami", "-d", "gitr"])
+        self.assertEqual(out, "Test User <some@example.com>\n")
+        self.assertEqual(err, "")
+
+        self.build_tree_contents([('gitr/.git/config', """\
+[user]
+  email = some@example.com
+""")])
+        out, err = self.run_bzr(["whoami", "-d", "gitr"])
+        self.assertEqual(out, "some@example.com\n")
+        self.assertEqual(err, "")
+
+    def test_local_signing_key(self):
+        r = GitRepo.init("gitr", mkdir=True)
+        self.build_tree_contents([('gitr/.git/config', """\
+[user]
+  email = some@example.com
+  name = Test User
+  signingkey = D729A457
+""")])
+        out, err = self.run_bzr(["config", "-d", "gitr", "gpg_signing_key"])
+        self.assertEqual(out, "D729A457\n")
+        self.assertEqual(err, "")
+
 
 class ShallowTests(ExternalBase):
 

=== modified file 'breezy/library_state.py'
--- breezy/library_state.py	2020-02-18 01:57:45 +0000
+++ breezy/library_state.py	2020-03-22 01:35:14 +0000
@@ -98,8 +98,9 @@
         self._trace.__enter__()
 
         self._orig_ui = breezy.ui.ui_factory
-        breezy.ui.ui_factory = self._ui
-        self._ui.__enter__()
+        if self._ui is not None:
+            breezy.ui.ui_factory = self._ui
+            self._ui.__enter__()
 
         if breezy._global_state is not None:
             raise RuntimeError("Breezy already initialized")
@@ -115,7 +116,8 @@
         trace._flush_stdout_stderr()
         trace._flush_trace()
         osutils.report_extension_load_failures()
-        self._ui.__exit__(None, None, None)
+        if self._ui is not None:
+            self._ui.__exit__(None, None, None)
         self._trace.__exit__(None, None, None)
         ui.ui_factory = self._orig_ui
         breezy._global_state = None

=== modified file 'breezy/plugins/propose/gitlabs.py'
--- breezy/plugins/propose/gitlabs.py	2020-03-07 15:47:50 +0000
+++ breezy/plugins/propose/gitlabs.py	2020-03-22 01:35:14 +0000
@@ -97,6 +97,10 @@
         self.error = error
 
 
+class MergeRequestExists(Exception):
+    """Raised when a merge requests already exists."""
+
+
 def default_config_path():
     return os.path.join(bedding.config_dir(), 'gitlab.conf')
 
@@ -401,7 +405,7 @@
         if response.status == 403:
             raise errors.PermissionDenied(response.text)
         if response.status == 409:
-            raise MergeProposalExists(self.source_branch.user_url)
+            raise MergeRequestExists()
         if response.status != 201:
             raise errors.InvalidHttpResponse(path, response.text)
         return json.loads(response.data)
@@ -616,7 +620,10 @@
                 else:
                     user = self.gl._get_user(reviewer)
                 kwargs['assignee_ids'].append(user['id'])
-        merge_request = self.gl._create_mergerequest(**kwargs)
+        try:
+            merge_request = self.gl._create_mergerequest(**kwargs)
+        except MergeRequestExists:
+            raise ProposalExists(self.source_branch.user_url)
         return GitLabMergeProposal(self.gl, merge_request)
 
 

=== modified file 'breezy/tests/test_library_state.py'
--- breezy/tests/test_library_state.py	2017-08-27 13:57:26 +0000
+++ breezy/tests/test_library_state.py	2020-02-21 22:54:31 +0000
@@ -52,3 +52,15 @@
         finally:
             state.__exit__(None, None, None)
             self.assertEqual(['__enter__', '__exit__'], tracer._calls)
+
+    def test_ui_not_specified(self):
+        self.overrideAttr(breezy, '_global_state', None)
+        state = library_state.BzrLibraryState(
+            ui=None, trace=fixtures.RecordingContextManager())
+        orig_ui = _mod_ui.ui_factory
+        state.__enter__()
+        try:
+            self.assertEqual(orig_ui, _mod_ui.ui_factory)
+        finally:
+            state.__exit__(None, None, None)
+            self.assertEqual(orig_ui, _mod_ui.ui_factory)

=== modified file 'doc/en/release-notes/brz-3.1.txt'
--- doc/en/release-notes/brz-3.1.txt	2020-02-08 17:00:10 +0000
+++ doc/en/release-notes/brz-3.1.txt	2020-03-21 20:56:33 +0000
@@ -114,6 +114,9 @@
  * Python 3 is now used by default to run scripts, etc. from the makefile.
    (Jelmer Vernooĳ)
 
+ * ``.git/config`` is now consulted to determine the users' identity
+   for commits, and the gpg_signing_key. (Jelmer Vernooĳ)
+
 Bug Fixes
 *********
 
@@ -148,6 +151,9 @@
 * Ignore ghost tags when interacting with remote Git repositories.
   (Jelmer Vernooĳ)
 
+* Fix ``setup_ui=False`` when initializing Breezy.
+  (Jelmer Vernooĳ, #1852647)
+
 Documentation
 *************
 

=== modified file 'doc/en/tutorials/using_breezy_with_github.txt'
--- doc/en/tutorials/using_breezy_with_github.txt	2018-11-22 03:04:59 +0000
+++ doc/en/tutorials/using_breezy_with_github.txt	2020-03-21 17:47:11 +0000
@@ -39,19 +39,25 @@
 GitHub is a code hosting service so you can push your
 branch to it and others can access your code from there.
 
-In the GitHub web UI, you can create a "fork" of an upstream repository. For
-example, if user "jelmer" forked the inkscape repository at
-https://github.com/inkscape/inkscape then the fork would be at
-https://github.com/jelmer/inkscape.
-
-You can then push your changes to a branch like this::
-
-  brz push git+ssh://github.com/jelmer/inkscape,branch=my-branch
-
-Others can then download your code like this::
-
-  brz branch git://github.com/jelmer/inkscape,branch=my-branch
-
+The first time you use GitHub, you will need to log into to your GitHub account
+with Breezy. This can be done by running ``brz github-login``. This will prompt
+you for your password, and store the authentication token in ~/.config/breezy.
+
+You can now use either ``brz publish`` to upload the changes back to
+GitHub, or ``brz propose`` to upload to the changes to GitHub and
+create merge proposal.
+
+Both ``brz publish`` and ``brz propose`` will create a fork of the upstream
+repository and then push your local changes to the active branch
+(or another branch specify with the --name argument).
+
+For example, running ``brz publish --name=my-branch`` in your local inkscape
+clone will create https://github.com/jelmer/inkscape if your GitHub username is
+"jelmer", and it will open your editor to allow you to enter the merge proposal
+description.
+
+Subsequent use of ``brz publish`` in the local branch (e.g. to push more
+commits) will update the previously created branch.
 
 Associating branches with GitHub issues
 ---------------------------------------

